--[[
this lib adds a scanning tooltip to get names of quests by ID, something the wow api does not
natively support.
for now, this lib will behave rather "strict", e.g. update and get will not automatically add
the quest and so on.
--]]

if(ThingsToDo2 == nil) then
	ThingsToDo2 = {}
end
local ttd2 = ThingsToDo2

if(ttd2.lib == nil) then
	ttd2.lib = {}
end
local lib = ttd2.lib

--this module
lib.quest = {}
local this = lib.quest

this.private = { set = false }

function this.private.Validate()
	if(this.private.set == false) then
		this.private.Init()
	end
	return
end

function this.private.Init()
	if(this.private.set) then
		return
	end
	
	--init tooltip parser
	this.private.tooltipParser = {}
	this.private.tooltipParser.MyScanningTooltip = CreateFrame("GameTooltip", "MyScanningTooltip", UIParent, "GameTooltipTemplate")
	this.private.tooltipParser.QuestTitleFromID = setmetatable({}, { __index = function(t, ID)
			this.private.tooltipParser.MyScanningTooltip:SetOwner(UIParent, "ANCHOR_NONE")
			this.private.tooltipParser.MyScanningTooltip:SetHyperlink("quest:"..ID)
			local title = MyScanningTooltipTextLeft1:GetText()
			this.private.tooltipParser.MyScanningTooltip:Hide()
			if title and title ~= RETRIEVING_DATA then
				t[ID] = title
				return title
			end
		end })
	
	this.private.data = {}
	this.private.incomplete = {}
	
	this.private.set = true
end

function this.private.ConstructLink(ID, Name)
	this.private.Validate()
	
	if(Name == nil) then
		Name = "quest:"..ID
	end
	
	--TODO: currently level 110 is hardcoded, may be a problem later
	local LinkParts = {}
	LinkParts[1] = "\124cFFFFFF00\124Hquest:"
	LinkParts[2] = ":110\124h["
	LinkParts[3] = "]\124h\124r"
	
	return (LinkParts[1]..ID..LinkParts[2]..Name..LinkParts[3])
end

function this.AddByID(ID, StoreObjectives)
	this.private.Validate()
	
	--adding the same quest multiple times must be allowed because different modules might do it
	if(this.private.data[ID] ~= nil) then
		return
	end
	
	this.private.data[ID] = 
	{
		Name = nil,
		Link = nil,
		isCompleted = nil,
		isInLog = nil
	}
	
	if(StoreObjectives == true) then
		this.private.data[ID].Objectives = {}
	end
	
	this.private.incomplete[ID] = true
	this.UpdateByID(ID)
	return
end

function this.UpdateByID(ID)
	this.private.Validate()
	
	if(this.private.data[ID] == nil) then
		lib.chat.error("trying to update quest before adding it: "..ID)
		return
	end
	
	if(this.private.incomplete[ID] ~= nil) then
		local complete = true
		local name = this.private.tooltipParser.QuestTitleFromID[ID]
			
		if((name == nil) or (name == " ")) then --there is currently a bug with empty questnames
			this.private.data[ID].Name = ("quest:"..ID)
			complete = false
		else
			this.private.data[ID].Name = name
		end
	
		this.private.data[ID].Link = this.private.ConstructLink(ID, this.private.data[ID].Name)
		
		if(complete) then
			this.private.incomplete[ID] = nil
		end
	end
	
	--this can change and always has to be updated
	this.private.data[ID].isCompleted = IsQuestFlaggedCompleted(ID)
	this.private.data[ID].isInLog = (GetQuestLogIndexByID(ID) ~= 0)
	
	if(this.private.data[ID].Objectives ~= nil) then
		lib.table.clear(this.private.data[ID].Objectives)

		local Index = 1
		local Objective = GetQuestObjectiveInfo(ID,Index,this.private.data[ID].isCompleted)
		while (Objective ~= nil) do
			this.private.data[ID].Objectives[Index] = Objective
			Index = Index+1
			Objective = GetQuestObjectiveInfo(ID,Index,this.private.data[ID].isCompleted)
		end
	end

	return
end

function this.GetByID(ID)
	this.private.Validate()
	
	if(this.private.data[ID] == nil) then
		lib.chat.error("trying to access quest before adding it: "..ID)
		return
	end
	
	return this.private.data[ID]
end
