if(ThingsToDo2 == nil) then
	ThingsToDo2 = {}
end
local ttd2 = ThingsToDo2

if(ttd2.lib == nil) then
	ttd2.lib = {}
end
local lib = ttd2.lib

if(ttd2.option == nil) then
	ttd2.option = {}
end
local option = ttd2.option

if(ttd2.mod == nil) then
	ttd2.mod = {}
end
local mod = ttd2.mod

--this module
ttd2.ArgusEgg = {}
local this = ttd2.ArgusEgg
this.Name = "ArgusEgg"

this.private = { set = false }

function this.private.Validate()
	if(this.private.set == false) then
		this.private.Init()
	end
	
	return
end

function this.private.Init()
	if(this.private.set) then
		return
	end
	
	this.private.Egg =
	{
		Item = { NormalID = 153190, CrackedID = 153191 },
		Content = 
		{
			Mounts =
			{
				{ ItemID = 152840, JournalID = 976 },
				{ ItemID = 152841, JournalID = 975 },
				{ ItemID = 152842, JournalID = 974 },
				{ ItemID = 152843, JournalID = 906 }
			},
			Pets =
			{
				{ ItemID = 153054, SpeciesID = 2118 },
				{ ItemID = 153055, SpeciesID = 2119 }
			}
		},
		SRC =
		{
			{ MapID = 830, PosX = 0.705, PosY = 0.337, FlagQuest = 48667, WQuest = 48502, Name = "Naroua" },	--Naroua
			{ MapID = 882, PosX = 0.442, PosY = 0.498, FlagQuest = 48712, WQuest = 48732, Name = "Sabuul" },	--Sabuul
			{ MapID = 885, PosX = 0.643, PosY = 0.482, FlagQuest = 48812, WQuest = 48827, Name = "Varga" }	--Varga
		}
	}
	
	lib.item.AddByID(this.private.Egg.Item.NormalID)
	lib.item.AddByID(this.private.Egg.Item.CrackedID)
	
	for _, Mount in pairs(this.private.Egg.Content.Mounts) do
		lib.item.AddByID(Mount.ItemID)
		lib.journal.mount.AddByID(Mount.JournalID)
	end
	
	for _, Pet in pairs(this.private.Egg.Content.Pets) do
		lib.item.AddByID(Pet.ItemID)
		lib.journal.pet.AddByID(Pet.SpeciesID)
	end
	
	for _, SRC in pairs(this.private.Egg.SRC) do
		lib.quest.AddByID(SRC.FlagQuest)
	end
	
	return
end

function this.Update()
	this.private.Validate()
	
	local SomethingMissing = false
	local SourcesAvailable = false
	
	lib.item.UpdateByID(this.private.Egg.Item.NormalID)
	lib.item.UpdateByID(this.private.Egg.Item.CrackedID)
	
	for _, Mount in pairs(this.private.Egg.Content.Mounts) do
		lib.item.UpdateByID(Mount.ItemID)
		lib.journal.mount.UpdateByID(Mount.JournalID)
		if(lib.journal.mount.GetByID(Mount.JournalID).isCollected ~= true) then
			SomethingMissing = true
		end
	end
	
	for _, Pet in pairs(this.private.Egg.Content.Pets) do
		lib.item.UpdateByID(Pet.ItemID)
		lib.journal.pet.UpdateByID(Pet.SpeciesID)
		if(lib.journal.pet.GetByID(Pet.SpeciesID).isCollected ~= true) then
			SomethingMissing = true
		end
	end
	
	for _, SRC in pairs(this.private.Egg.SRC) do
		lib.quest.UpdateByID(SRC.FlagQuest)
		if(lib.quest.GetByID(SRC.FlagQuest).isCompleted ~= true) then
			SourcesAvailable = true
		end
		SRC.Name = lib.worldquest.GetNameByID(SRC.WQuest)
	end
	
	local out = lib.chat.output
	out.ClearBuffer()
	
	local EggLink = lib.item.GetByID(this.private.Egg.Item.NormalID).Link.." >> "..lib.item.GetByID(this.private.Egg.Item.CrackedID).Link
	if(SomethingMissing == false) then
		out.StoreCompleted(EggLink)
	elseif(SourcesAvailable == false) then
		out.StoreDone(lib.item.GetByID(EggLink))
	else
		out.StoreTodo(lib.item.GetByID(EggLink))
	end
	
	for _, Mount in pairs(this.private.Egg.Content.Mounts) do
		if(lib.journal.mount.GetByID(Mount.JournalID).isCollected ~= true) then
			if(SourcesAvailable ~= true) then
				out.StoreDone(lib.item.GetByID(Mount.ItemID).Link)
			else
				out.StoreTodo(lib.item.GetByID(Mount.ItemID).Link)
			end			
		else
			out.StoreCompleted(lib.item.GetByID(Mount.ItemID).Link)
		end
	end

	--for pets, Done has priority over completed, because people might want to farm duplicates
	for _, Pet in pairs(this.private.Egg.Content.Pets) do
		local msg = lib.item.GetByID(Pet.ItemID).Link
		if(SourcesAvailable ~= true) then
			out.StoreDone(msg)
		else
			local PetData = lib.journal.pet.GetByID(Pet.SpeciesID)
			msg = msg.." ("..PetData.NumCollected.." / "..PetData.NumLimit..")"
			if(lib.journal.pet.GetByID(Pet.SpeciesID).isCollected ~= true) then
				out.StoreTodo(msg)
			else
				out.StoreCompleted(msg)
			end
		end
	end
	
	--now list sources; as these effect pets, done takes priority again
	for _, SRC in pairs(this.private.Egg.SRC) do
		if(lib.quest.GetByID(SRC.FlagQuest).isCompleted ~= true) then
			local msg = lib.tomtom.createLink(SRC.MapID,SRC.PosX,SRC.PosY, SRC.Name)
			if(lib.worldquest.IsActive(SRC.WQuest) == true) then
				msg = msg..lib.chat.yellow(" WQ active")
			else
				msg = msg..lib.chat.red(" WQ not active")
			end
			
			if(SomethingMissing == true) then
				out.StoreTodo(msg)
			else
				out.StoreCompleted(msg)
			end
		else
			out.StoreDone(SRC.Name)
		end
	end
	
	return
end

function this.Print()
	this.private.Validate()
	
	local out = lib.chat.output
	out.PrintModule(this.Name)
	
	return
end

this.private.Text = this.Name
function this.private.Script()
	this.private.Validate()
	
	this.Update()
	this.Print()
	
	return
end

mod[#mod+1] =
{
	Text = this.private.Text,
	Script = this.private.Script
}
